import uvicorn
import requests
from typing import Annotated
from pydantic import BaseModel
from datetime import datetime
from sqlalchemy.orm import Session
from fastapi import FastAPI, Body, Depends, HTTPException

from . import crud, models, schemas
from .database import SessionLocal, engine
from .docs import sendMessageExamples

models.Base.metadata.create_all(bind=engine)

app = FastAPI()

def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()

@app.get("/")
def read_root():
    return {"Made by": "Nikurasu", "Version": "0.0.0"}



@app.post("/send_message", response_model=schemas.TelegramBotMessage)
def send_message(body: Annotated[schemas.SendMessagePostBody, Body(openapi_examples=sendMessageExamples)], db: Session = Depends(get_db)):
    tg_url = f"https://api.telegram.org/bot{body.bot_token}/sendMessage"
    tg_body = {"chat_id": body.chat_id, "text": body.message}
    tg_result = requests.post(url=tg_url, json=tg_body)
    if tg_result.status_code != 200:
        raise HTTPException(status_code=500, detail=f"internal server error")
    res_json = tg_result.json()
    chat_id_from_result = res_json["result"]["chat"]["id"]
    message_id_from_result = res_json["result"]["message_id"]
    return crud.create_TelegramBotMessage(db=db, telegram_bot_message=schemas.TelegramBotMessageCreate(
        chat_id=str(chat_id_from_result),
        message_id=str(message_id_from_result),
        datetime=datetime.now()
    ))

if __name__ == "__main__":
    uvicorn.run(app, host="0.0.0.0", port=8080)